@@ -37,6 +37,7 @@
 #include <panel-applet-gconf.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "cpufreq-applet.h"
@@ -66,7 +67,12 @@ struct _CPUFreqApplet {
         GtkWidget        *box;
         GtkWidget        *container;
         GdkPixbuf        *pixbufs[5];
-        GtkTooltips      *tips;
+
+	gint              max_label_width;
+	gint              max_perc_width;
+	gint              max_unit_width;
+
+	gboolean          need_refresh;
 
         CPUFreqPrefs     *prefs;
         CPUFreqPopup     *popup;
@@ -104,6 +110,8 @@ static gboolean cpufreq_applet_key_press         (GtkWidget          *widget,
                                                   GdkEventKey        *event);
 static void     cpufreq_applet_size_allocate     (GtkWidget          *widget,
                                                   GtkAllocation      *allocation);
+static void     cpufreq_applet_size_request      (GtkWidget          *widget,
+						  GtkRequisition     *requisition);
 static void     cpufreq_applet_change_orient     (PanelApplet        *pa,
                                                   PanelAppletOrient   orient);
 static void     cpufreq_applet_change_background (PanelApplet        *pa,
@@ -175,30 +183,9 @@ cpufreq_applet_show_text_mode_get_type (void)
         return etype;
 }
 
-GType
-cpufreq_applet_selector_mode_get_type (void)
-{
-        static GType etype = 0;
-
-        if (etype == 0) {
-                static const GEnumValue values[] = {
-                        { CPUFREQ_SELECTOR_MODE_FREQUENCIES, "CPUFREQ_SELECTOR_MODE_FREQUENCIES", "selector-mode-frequencies" },
-                        { CPUFREQ_SELECTOR_MODE_GOVERNORS,   "CPUFREQ_SELECTOR_MODE_GOVERNORS",   "selector-mode-governors" },
-                        { CPUFREQ_SELECTOR_MODE_BOTH,        "CPUFREQ_SELECTOR_MODE_BOTH",        "selector-mode-both" },
-                        { 0, NULL, NULL }
-                };
-
-                etype = g_enum_register_static ("CPUFreqSelectorMode", values);
-        }
-
-        return etype;
-}
-
 static void
 cpufreq_applet_init (CPUFreqApplet *applet)
 {
-        gint i;
-
         applet->prefs = NULL;
         applet->popup = NULL;
         applet->monitor = NULL;
@@ -207,21 +194,28 @@ cpufreq_applet_init (CPUFreqApplet *applet)
         applet->unit_label = gtk_label_new (NULL);
         applet->icon = gtk_image_new ();
         applet->box = NULL;
-           
-        applet->container = gtk_alignment_new (0.5, 0.5, 0, 0);
-        gtk_container_add (GTK_CONTAINER (applet), applet->container);
-        gtk_widget_show (applet->container);
-           
-        applet->tips = gtk_tooltips_new ();
-           
-        for (i = 0; i <= 4; i++)
-                applet->pixbufs[i] = NULL;
+
+	applet->need_refresh = TRUE;
 
         panel_applet_set_flags (PANEL_APPLET (applet), PANEL_APPLET_EXPAND_MINOR);
 
         applet->size = panel_applet_get_size (PANEL_APPLET (applet));
         applet->orient = panel_applet_get_orient (PANEL_APPLET (applet));
 
+	switch (applet->orient) {
+	case PANEL_APPLET_ORIENT_LEFT:
+	case PANEL_APPLET_ORIENT_RIGHT:
+		applet->container = gtk_alignment_new (0.5, 0.5, 0, 0);
+		break;
+	case PANEL_APPLET_ORIENT_UP:
+	case PANEL_APPLET_ORIENT_DOWN:
+		applet->container = gtk_alignment_new (0, 0.5, 0, 0);
+		break;
+	}
+
+	gtk_container_add (GTK_CONTAINER (applet), applet->container);
+	gtk_widget_show (applet->container);
+
         gtk_widget_show (GTK_WIDGET (applet));
 }
 
@@ -235,6 +229,7 @@ cpufreq_applet_class_init (CPUFreqAppletClass *klass)
         gtkobject_class->destroy = cpufreq_applet_destroy;
            
         widget_class->size_allocate = cpufreq_applet_size_allocate;
+	widget_class->size_request = cpufreq_applet_size_request;
         widget_class->button_press_event = cpufreq_applet_button_press;
         widget_class->key_press_event = cpufreq_applet_key_press;
            
@@ -255,11 +250,6 @@ cpufreq_applet_destroy (GtkObject *widget)
                 applet->monitor = NULL;
         }
 
-        if (applet->tips) {
-                gtk_object_destroy (GTK_OBJECT (applet->tips));
-                applet->tips = NULL;
-        }
-
         for (i = 0; i <= 3; i++) {
                 if (applet->pixbufs[i]) {
                         g_object_unref (G_OBJECT (applet->pixbufs[i]));
@@ -307,6 +297,124 @@ cpufreq_applet_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
         GTK_WIDGET_CLASS (cpufreq_applet_parent_class)->size_allocate (widget, allocation);
 }
 
+static gint
+cpufreq_applet_get_max_label_width (CPUFreqApplet *applet)
+{
+	GList *available_freqs;
+	gint   width = 0;
+	
+	if (applet->max_label_width > 0)
+		return applet->max_label_width;
+
+	if (!CPUFREQ_IS_MONITOR (applet->monitor))
+		return 0;
+
+	available_freqs = cpufreq_monitor_get_available_frequencies (applet->monitor);
+	while (available_freqs) {
+		GtkWidget     *label;
+		GtkRequisition req;
+		const gchar   *text;
+		gchar         *freq_text;
+		gint           freq;
+
+		text = (const gchar *) available_freqs->data;
+		freq = atoi (text);
+
+		freq_text = cpufreq_utils_get_frequency_label (freq);
+		label = gtk_label_new (freq_text);
+		gtk_widget_size_request (label, &req);
+
+		width = MAX (width, req.width);
+
+		g_free (freq_text);
+		gtk_widget_destroy (label);
+
+		available_freqs = g_list_next (available_freqs);
+	}
+
+	applet->max_label_width = width;
+
+	return width;
+}
+
+static gint
+cpufreq_applet_get_max_perc_width (CPUFreqApplet *applet)
+{
+	GtkWidget      *label;
+	GtkRequisition req;
+	
+	if (applet->max_perc_width > 0)
+		return applet->max_perc_width;
+
+	label = gtk_label_new ("100%");
+	gtk_widget_size_request (label, &req);
+	applet->max_perc_width = req.width;
+	gtk_widget_destroy (label);
+
+	return applet->max_perc_width;
+}
+
+static gint
+cpufreq_applet_get_max_unit_width (CPUFreqApplet *applet)
+{
+	GtkWidget     *label;
+	GtkRequisition req;
+	gint           w1, w2;
+	
+	if (applet->max_unit_width > 0)
+		return applet->max_unit_width;
+
+	label = gtk_label_new ("GHz");
+	gtk_widget_size_request (label, &req);
+	w1 = req.width;
+
+	gtk_label_set_text (GTK_LABEL (label), "MHz");
+	gtk_widget_size_request (label, &req);
+	w2 = req.width;
+	
+	gtk_widget_destroy (label);
+
+	applet->max_unit_width = MAX (w1, w2);
+
+	return applet->max_unit_width;
+}
+
+static void
+cpufreq_applet_size_request (GtkWidget *widget, GtkRequisition *requisition)
+{
+	CPUFreqApplet *applet;
+	gint           width = 0;
+
+	applet = CPUFREQ_APPLET (widget);
+	
+	GTK_WIDGET_CLASS (cpufreq_applet_parent_class)->size_request (widget, requisition);
+
+	if (applet->orient == PANEL_APPLET_ORIENT_LEFT ||
+	    applet->orient == PANEL_APPLET_ORIENT_RIGHT)
+		return;
+	
+	if (applet->show_freq) {
+		width += cpufreq_applet_get_max_label_width (applet) + 2;
+        }
+
+        if (applet->show_perc) {
+		width += cpufreq_applet_get_max_perc_width (applet);
+        }
+
+        if (applet->show_unit) {
+		width += cpufreq_applet_get_max_unit_width (applet);
+        }
+
+        if (applet->show_icon) {
+		GtkRequisition req;
+		
+		gtk_widget_size_request (applet->icon, &req);
+		width += req.width + 2;
+        }
+
+	requisition->width = width;
+}
+
 static void
 cpufreq_applet_popup_position_menu (GtkMenu  *menu,
                                     int      *x,
@@ -363,7 +471,6 @@ cpufreq_applet_menu_popup (CPUFreqApplet *applet,
 
         if (!applet->popup) {
                 applet->popup = cpufreq_popup_new ();
-                cpufreq_popup_set_preferences (applet->popup, applet->prefs);
                 cpufreq_popup_set_monitor (applet->popup, applet->monitor);
         }
 
@@ -436,8 +543,12 @@ cpufreq_applet_change_orient (PanelApplet *pa, PanelAppletOrient orient)
         if ((orient == PANEL_APPLET_ORIENT_LEFT) ||
             (orient == PANEL_APPLET_ORIENT_RIGHT)) {
                 size = GTK_WIDGET (applet)->allocation.width;
+		gtk_alignment_set (GTK_ALIGNMENT (applet->container),
+				   0.5, 0.5, 0, 0);
         } else {
                 size = GTK_WIDGET (applet)->allocation.height;
+		gtk_alignment_set (GTK_ALIGNMENT (applet->container),
+				   0, 0.5, 0, 0);
         }
 
         if (size != applet->size) {
@@ -532,7 +643,6 @@ cpufreq_applet_about_cb (BonoboUIComponent *uic,
         };
 
         gtk_show_about_dialog (NULL,
-                               "name",          _("CPU Frequency Scaling Monitor"),
                                "version",       VERSION,
                                "copyright",     "\xC2\xA9 2004 Carlos Garcia Campos",
                                "comments",      _("This utility shows the current CPU "
@@ -663,12 +773,11 @@ cpufreq_applet_update (CPUFreqApplet *applet, CPUFreqMonitor *monitor)
         gint            perc;
         guint           cpu;
         gchar          *governor;
-        static gboolean do_refresh = TRUE;
 
         cpu = cpufreq_monitor_get_cpu (monitor);
         freq = cpufreq_monitor_get_frequency (monitor);
         perc = cpufreq_monitor_get_percentage (monitor);
-        governor = cpufreq_monitor_get_governor (monitor);
+        governor = g_strdup (cpufreq_monitor_get_governor (monitor));
 
         freq_label = cpufreq_utils_get_frequency_label (freq);
         unit_label = cpufreq_utils_get_frequency_unit (freq);
@@ -709,13 +818,13 @@ cpufreq_applet_update (CPUFreqApplet *applet, CPUFreqMonitor *monitor)
 
         g_free (text_mode);
            
-        gtk_tooltips_set_tip (applet->tips, GTK_WIDGET (applet), text_tip, NULL);
+        gtk_widget_set_tooltip_text (GTK_WIDGET (applet), text_tip);
         g_free (text_tip);
 
         /* Call refresh only the first time */
-        if (do_refresh) {
+        if (applet->need_refresh) {
                 cpufreq_applet_refresh (applet);
-                do_refresh = FALSE;
+		applet->need_refresh = FALSE;
         }
 }
 
@@ -745,7 +854,7 @@ cpufreq_applet_get_widget_size (CPUFreqApplet *applet,
         }
 
         return size;
-}
+} 
 
 static void
 cpufreq_applet_refresh (CPUFreqApplet *applet)
@@ -755,19 +864,27 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
         gint       panel_size, label_size;
         gint       unit_label_size, pixmap_size;
         gint       size_step = 12;
+	gboolean   horizontal;
         gboolean   do_unref = FALSE;
 
         panel_size = applet->size - 1; /* 1 pixel margin */
 
+	horizontal = (applet->orient == PANEL_APPLET_ORIENT_UP ||
+		      applet->orient == PANEL_APPLET_ORIENT_DOWN);
+	
         /* We want a fixed label size, the biggest */
-        gtk_label_set_width_chars (GTK_LABEL (applet->label), 4);
-        label_size = cpufreq_applet_get_widget_size (applet, applet->label);
-        gtk_label_set_width_chars (GTK_LABEL (applet->label), -1);
+	if (horizontal)
+		label_size = cpufreq_applet_get_widget_size (applet, applet->label);
+	else
+		label_size = cpufreq_applet_get_max_label_width (applet);
         total_size += label_size;
-           
-        unit_label_size = cpufreq_applet_get_widget_size (applet, applet->unit_label);
+
+	if (horizontal)
+		unit_label_size = cpufreq_applet_get_widget_size (applet, applet->unit_label);
+	else
+		unit_label_size = cpufreq_applet_get_max_unit_width (applet);
         total_size += unit_label_size;
-        
+
         pixmap_size = cpufreq_applet_get_widget_size (applet, applet->icon);
         total_size += pixmap_size;
 
@@ -791,9 +908,13 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
                 gtk_widget_destroy (applet->box);
         }
 
-        switch (applet->orient) {
-        case PANEL_APPLET_ORIENT_LEFT:
-        case PANEL_APPLET_ORIENT_RIGHT:
+	if (horizontal) {
+		labels_box = gtk_hbox_new (FALSE, 2);
+		if ((label_size + pixmap_size) <= panel_size)
+			applet->box = gtk_vbox_new (FALSE, 2);
+		else
+			applet->box = gtk_hbox_new (FALSE, 2);
+	} else {
                 if (total_size <= panel_size) {
                         applet->box = gtk_hbox_new (FALSE, 2);
                         labels_box  = gtk_hbox_new (FALSE, 2);
@@ -804,24 +925,13 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
                         applet->box = gtk_vbox_new (FALSE, 2);
                         labels_box  = gtk_vbox_new (FALSE, 2);
                 }
-                break;
-        case PANEL_APPLET_ORIENT_UP:
-        case PANEL_APPLET_ORIENT_DOWN:
-                labels_box = gtk_hbox_new (FALSE, 2);
-                if ((label_size + pixmap_size) <= panel_size)
-                        applet->box = gtk_vbox_new (FALSE, 2);
-                else
-                        applet->box = gtk_hbox_new (FALSE, 2);
-                break;
-        default:
-                g_assert_not_reached ();
-        }
+	}
 
-        gtk_box_pack_start (GTK_BOX (labels_box), applet->label, FALSE, TRUE, 0);
-        gtk_box_pack_start (GTK_BOX (labels_box), applet->unit_label, FALSE, TRUE, 0);
-        gtk_box_pack_start (GTK_BOX (applet->box), applet->icon, FALSE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (labels_box), applet->label, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (labels_box), applet->unit_label, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (applet->box), applet->icon, FALSE, FALSE, 0);
         
-        gtk_box_pack_start (GTK_BOX (applet->box), labels_box, FALSE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (applet->box), labels_box, FALSE, FALSE, 0);
         gtk_widget_show (labels_box);
            
         gtk_container_add (GTK_CONTAINER (applet->container), applet->box);
@@ -835,6 +945,15 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
 }
 
 /* Preferences callbacks */
+static void
+cpufreq_applet_prefs_cpu_changed (CPUFreqPrefs  *prefs,
+				  GParamSpec    *arg1,
+				  CPUFreqApplet *applet)
+{
+	cpufreq_monitor_set_cpu (applet->monitor,
+				 cpufreq_prefs_get_cpu (applet->prefs));
+}
+
 static void
 cpufreq_applet_prefs_show_mode_changed (CPUFreqPrefs  *prefs,
                                         GParamSpec    *arg1,
@@ -850,7 +969,7 @@ cpufreq_applet_setup (CPUFreqApplet *applet)
         AtkObject         *atk_obj;
         gchar             *prefs_key;
 
-        glade_gnome_init ();
+	g_set_application_name  (_("CPU Frequency Scaling Monitor"));
 
 	gtk_window_set_default_icon_name ("gnome-cpu-frequency-applet");
 	
@@ -865,6 +984,10 @@ cpufreq_applet_setup (CPUFreqApplet *applet)
         applet->prefs = cpufreq_prefs_new (prefs_key);
         g_free (prefs_key);
 
+	g_signal_connect (G_OBJECT (applet->prefs),
+			  "notify::cpu",
+			  G_CALLBACK (cpufreq_applet_prefs_cpu_changed),
+			  (gpointer) applet);
         g_signal_connect (G_OBJECT (applet->prefs),
                           "notify::show-mode",
                           G_CALLBACK (cpufreq_applet_prefs_show_mode_changed),
