@@ -2,7 +2,7 @@ import os, time
 from os.path import *
 import gnomeapplet, gtk, gtk.gdk, gconf, gobject
 from gettext import gettext as _
-import gtk, gtk.glade, egg.trayicon, gobject, gnomevfs
+import gtk, gtk.glade, gobject, gnomevfs
 import csv, os
 
 import invest, invest.about, invest.chart
@@ -53,7 +53,10 @@ class QuoteUpdater(gtk.ListStore):
 			result[fields[0]] = {}
 			for i, field in enumerate(invest.QUOTES_CSV_FIELDS):
 				if type(field) == tuple:
-					result[fields[0]][field[0]] = field[1](fields[i])
+					try:
+						result[fields[0]][field[0]] = field[1](fields[i])
+					except:
+						result[fields[0]][field[0]] = 0
 				else:
 					result[fields[0]][field] = fields[i]
 					
@@ -62,14 +65,10 @@ class QuoteUpdater(gtk.ListStore):
 	def populate(self, quotes):
 		self.clear()
 		
-		#current = sum([sum([purchase["amount"]*val["trade"] for purchase in invest.STOCKS[ticker]]) for ticker, val in quotes.items()])
-		#paid = sum([sum([purchase["amount"]*purchase["bought"]+purchase["comission"] for purchase in invest.STOCKS[ticker]]) for ticker in quotes.keys()])
-		#balance = current - paid	
-				
-		#self.append([_("Total"), balance, balance/paid*100, current, 0])
-		
-		for ticker, val in quotes.items():
-			# Check whether the symbol is a syimple quote, or a portfolio value
+		quote_items = quotes.items ()
+		quote_items.sort ()
+		for ticker, val in quote_items:
+			# Check whether the symbol is a simple quote, or a portfolio value
 			is_simple_quote = True
 			for purchase in invest.STOCKS[ticker]:
 				if purchase["amount"] != 0:
@@ -82,8 +81,11 @@ class QuoteUpdater(gtk.ListStore):
 				current = sum([purchase["amount"]*val["trade"] for purchase in invest.STOCKS[ticker] if purchase["amount"] != 0])
 				paid = sum([purchase["amount"]*purchase["bought"] + purchase["comission"] for purchase in invest.STOCKS[ticker] if purchase["amount"] != 0])
 				balance = current - paid
-				
-				self.append([ticker, False, balance, balance/paid*100, val["trade"], val["variation"]])
+				if paid != 0:
+					change = 100*balance/paid
+				else:
+					change = 100 # Not technically correct, but it will look more intuitive than the real result of infinity.
+				self.append([ticker, False, balance, change, val["trade"], val["variation"]])
 				
 		self.emit("quotes-updated")
 
